#include <algorithm>
#include <cstdio>
#include <cmath>
#include <array>
#include <map>

#define MIN_VAL (-1000000)
#define MAX_VAL (+1000000)
#define MAX_NUM (50000)
#define DIM (3)

typedef std::array<long *, DIM> ITuple;
static long DAT[DIM][MAX_NUM]; static long IND[DIM][MAX_NUM], TMP[2][MAX_NUM]; static char CHK[MAX_NUM];

inline long dis(long a, long b) {
	static long k, ans; for (ans = k = 0; k < DIM; ++ k) ans += std::abs(DAT[k][a] - DAT[k][b]);
	return ans;
}

long get_min_dis(ITuple DAT, ITuple IND, long num) {
	//...
	if (num == 2) return dis(IND[0][0], IND[0][1]); else if (num <= 1) return -1;
	//Выполняем поиск подходящей оси координат и гиперплоскости для разделения исходного множества:
	long p, k, l, r, nl, nr, k0, p0, v0, d0, dl, dr;
	for (d0 = k0 = k = 0; k < DIM; ++ k) {
		long b = DAT[k][IND[k][num - 1]], a = DAT[k][IND[k][0]], d = b - a;
		if (d0 < d) { d0 = d; k0 = k; }
	}
	std::swap(DAT[k0], DAT[0]); std::swap(IND[k0], IND[0]);
	//Разделяем множество на две части относительно выбранной гиперплоскости:
	nl = 0; nr = 0; p0 = num / 2; v0 = DAT[0][IND[0][p0]];
	for (p = 0; p < num; ++ p) {
		if (p > p0) { CHK[IND[0][p]] = false; ++ nr; }
		else { CHK[IND[0][p]] = true; ++ nl; }
	}
	for (k = 0; k < DIM; ++ k) {
		for (l = r = p = 0; p < num; ++ p) {
			if (CHK[IND[k][p]]) { TMP[0][l ++] = IND[k][p]; }
			else { TMP[1][r ++] = IND[k][p]; }
		}
		std::copy(TMP[1], TMP[1] + nr, IND[k] + nl);
		std::copy(TMP[0], TMP[0] + nl, IND[k]);
	}
	//Выполняем рекурсивный обход выделенных подмножеств:
	ITuple R{IND[0] + nl, IND[1] + nl, IND[2] + nl};
	ITuple L{IND[0], IND[1], IND[2]};
	dl = get_min_dis(DAT, L, nl);
	dr = get_min_dis(DAT, R, nr);
	//Восстанавливаем сортировку по координатам:
	for (k = 0; k < DIM; ++ k) {
		std::merge(L[k], L[k] + nl, R[k], R[k] + nr, TMP[0],
		          [DAT, k] (long a, long b) {
				  return DAT[k][a] < DAT[k][b];
		});
		std::copy(TMP[0], TMP[0] + nl + nr, IND[k]);
	}
	//...
	if ((dl == -1) || (dr == -1))
		d0 = std::max(dl, dr);
	else {
		d0 = std::min(dl, dr);
	}
	if (d0 == 0) return d0;
	//Выделяем d-слой точек, лежащих слева и справа от гиперплоскости:
	for (l = 0; l < nl; ++ l) CHK[L[0][l]] = false;
	for (r = 0; r < nr; ++ r) CHK[R[0][r]] = true;
	//Строим разреженные прямоугольные сетки для обоих подмножеств:
	std::map<std::pair<long, long>, long> LGRID, RGRID;
	static long i1, i2, NEXT[MAX_NUM];
	std::fill(NEXT, NEXT + num, -1);
	for (p = 0; p < num; ++ p) {
		long k = IND[0][p]; if (std::abs(DAT[0][k] - v0) > d0) continue;
		i1 = (DAT[1][k] - MIN_VAL) / d0;
		i2 = (DAT[2][k] - MIN_VAL) / d0;
		auto &dic = CHK[k]? (RGRID): (LGRID);
		auto key = std::make_pair(i1, i2);
		auto it = dic.find(key);
		if (it != dic.end()) {
			NEXT[p] = it->second; it->second = p; continue;
		}
		dic[key] = p;
	}
	//Выполняем поиск между двумя сетками:
	for (auto itL: LGRID) {
		auto key = itL.first;
		for (i2 = key.second - 1; i2 < key.second + 1; ++ i2)
		for (i1 = key.first - 1; i1 < key.first + 1; ++ i1) {
			auto itR = RGRID.find(std::make_pair(i1, i2));
			if (itR == RGRID.end()) continue;
			for (r = itR->second; r != -1; r = NEXT[r])
			for (l = itL.second; l != -1; l = NEXT[l]) {
				long d = dis(IND[0][r], IND[0][l]);
				d0 = std::min(d0, d);
			}
		}
	}
	//...
	return d0;
}

int main() {
	static long num, ans;
	FILE *fid = fopen("input.txt", "rt"); fscanf(fid, "%li", &num);
	for (long i = 0; i < num; ++ i) {
		fscanf(fid, "%li %li %li", &DAT[0][i], &DAT[1][i], &DAT[2][i]);
		IND[0][i] = IND[1][i] = IND[2][i] = i;
	}
	fclose(fid);
	//Предварительные сортировки:
	for (long k = 0; k < 3; ++ k) {
		std::sort(IND[k], IND[k] + num, [k] (long a, long b) {
			return DAT[k][a] < DAT[k][b];
		});
	}
	//...
	ITuple dat{DAT[0], DAT[1], DAT[2]};
	ITuple ind{IND[0], IND[1], IND[2]};
	ans = (get_min_dis(dat, ind, num) + 1) / 2;
	//...
	fid = fopen("output.txt", "wt");
	fprintf(fid, "%li\n", ans);
	fclose(fid);
	//...
	return 0;
}
